"""Checker that looks for invalid attributes passed to nextpy functions."""

# TODO: add tests for this
import difflib
from typing import TYPE_CHECKING, Optional

import astroid
from pylint.checkers import BaseChecker

if TYPE_CHECKING:
    from pylint.lint import PyLinter

# List of valid nextpy attributes
VALID_ATTRIBUTES = set(
    [
    "App",
    "UploadFile",
    "app",
    "AdminDash",
    "admin",
    "EventChain",
    "background",
    "call_script",
    "clear_local_storage",
    "console_log",
    "download",
    "event",
    "prevent_default",
    "redirect",
    "remove_cookie",
    "remove_local_storage",
    "set_clipboard",
    "set_focus",
    "set_value",
    "stop_propagation",
    "upload_files",
    "window_alert",
    "Middleware",
    "middleware",
    "route",
    "Cookie",
    "LocalStorage",
    "State",
    "state",
    "var",
    "Var",
    "cached_var",
    "vars",
    "Base",
    "base",
    "compiler",
    "get_asset_path",
    "Config",
    "DBConfig",
    "config",
    "testing",
    "Env",
    "constants",
    "JsonDatabase",
    "Model",
    "model",
    "session",
    "Accordion",
    "AccordionButton",
    "AccordionIcon",
    "AccordionItem",
    "AccordionPanel",
    "Alert",
    "AlertDescription",
    "AlertDialog",
    "AlertDialogBody",
    "AlertDialogContent",
    "AlertDialogFooter",
    "AlertDialogHeader",
    "AlertDialogOverlay",
    "AlertIcon",
    "AlertTitle",
    "AspectRatio",
    "Audio",
    "Avatar",
    "AvatarBadge",
    "AvatarGroup",
    "Badge",
    "Box",
    "Breadcrumb",
    "BreadcrumbItem",
    "BreadcrumbLink",
    "BreadcrumbSeparator",
    "Button",
    "ButtonGroup",
    "Card",
    "CardBody",
    "CardFooter",
    "CardHeader",
    "Center",
    "Checkbox",
    "CheckboxGroup",
    "CircularProgress",
    "CircularProgressLabel",
    "Circle",
    "Code",
    "CodeBlock",
    "Collapse",
    "ColorModeButton",
    "ColorModeIcon",
    "ColorModeSwitch",
    "Component",
    "Cond",
    "ConnectionBanner",
    "ConnectionModal",
    "Container",
    "DataTable",
    "DataEditor",
    "DataEditorTheme",
    "DatePicker",
    "DateTimePicker",
    "DebounceInput",
    "Divider",
    "Drawer",
    "DrawerBody",
    "DrawerCloseButton",
    "DrawerContent",
    "DrawerFooter",
    "DrawerHeader",
    "DrawerOverlay",
    "Editable",
    "EditableInput",
    "EditablePreview",
    "EditableTextarea",
    "Editor",
    "Email",
    "Fade",
    "Flex",
    "Foreach",
    "Form",
    "FormControl",
    "FormErrorMessage",
    "FormHelperText",
    "FormLabel",
    "Fragment",
    "Grid",
    "GridItem",
    "Heading",
    "Highlight",
    "Hstack",
    "Html",
    "Icon",
    "IconButton",
    "Image",
    "Input",
    "InputGroup",
    "InputLeftAddon",
    "InputLeftElement",
    "InputRightAddon",
    "InputRightElement",
    "Kbd",
    "Link",
    "LinkBox",
    "LinkOverlay",
    "List",
    "ListItem",
    "Markdown",
    "Menu",
    "MenuButton",
    "MenuDivider",
    "MenuGroup",
    "MenuItem",
    "MenuItemOption",
    "MenuList",
    "MenuOptionGroup",
    "Modal",
    "ModalBody",
    "ModalCloseButton",
    "ModalContent",
    "ModalFooter",
    "ModalHeader",
    "ModalOverlay",
    "Moment",
    "MultiSelect",
    "MultiSelectOption",
    "NextLink",
    "NumberDecrementStepper",
    "NumberIncrementStepper",
    "NumberInput",
    "NumberInputField",
    "NumberInputStepper",
    "Option",
    "OrderedList",
    "Password",
    "PinInput",
    "PinInputField",
    "Plotly",
    "Popover",
    "PopoverAnchor",
    "PopoverArrow",
    "PopoverBody",
    "PopoverCloseButton",
    "PopoverContent",
    "PopoverFooter",
    "PopoverHeader",
    "PopoverTrigger",
    "Progress",
    "Radio",
    "RadioGroup",
    "RangeSlider",
    "RangeSliderFilledTrack",
    "RangeSliderThumb",
    "RangeSliderTrack",
    "ResponsiveGrid",
    "ScaleFade",
    "Script",
    "Select",
    "Skeleton",
    "SkeletonCircle",
    "SkeletonText",
    "Slide",
    "SlideFade",
    "Slider",
    "SliderFilledTrack",
    "SliderMark",
    "SliderThumb",
    "SliderTrack",
    "Spacer",
    "Span",
    "Spinner",
    "Square",
    "Stack",
    "Stat",
    "StatArrow",
    "StatGroup",
    "StatHelpText",
    "StatLabel",
    "StatNumber",
    "Step",
    "StepDescription",
    "StepIcon",
    "StepIndicator",
    "StepNumber",
    "StepSeparator",
    "StepStatus",
    "StepTitle",
    "Stepper",
    "Switch",
    "Tab",
    "TabList",
    "TabPanel",
    "TabPanels",
    "Table",
    "TableCaption",
    "TableContainer",
    "Tabs",
    "Tag",
    "TagCloseButton",
    "TagLabel",
    "TagLeftIcon",
    "TagRightIcon",
    "Tbody",
    "Td",
    "Text",
    "TextArea",
    "Tfoot",
    "Th",
    "Thead",
    "Tooltip",
    "Tr",
    "UnorderedList",
    "Upload",
    "Video",
    "VisuallyHidden",
    "Vstack",
    "Wrap",
    "WrapItem",
    "accordion",
    "accordion_button",
    "accordion_icon",
    "accordion_item",
    "accordion_panel",
    "alert",
    "alert_description",
    "alert_dialog",
    "alert_dialog_body",
    "alert_dialog_content",
    "alert_dialog_footer",
    "alert_dialog_header",
    "alert_dialog_overlay",
    "alert_icon",
    "alert_title",
    "aspect_ratio",
    "audio",
    "avatar",
    "avatar_badge",
    "avatar_group",
    "badge",
    "box",
    "breadcrumb",
    "breadcrumb_item",
    "breadcrumb_link",
    "breadcrumb_separator",
    "button",
    "button_group",
    "card",
    "card_body",
    "card_footer",
    "card_header",
    "center",
    "checkbox",
    "checkbox_group",
    "circular_progress",
    "circular_progress_label",
    "circle",
    "code",
    "code_block",
    "collapse",
    "color_mode_button",
    "color_mode_icon",
    "color_mode_switch",
    "component",
    "cond",
    "connection_banner",
    "connection_modal",
    "container",
    "data_table",
    "data_editor",
    "data_editor_theme",
    "date_picker",
    "date_time_picker",
    "debounce_input",
    "divider",
    "drawer",
    "drawer_body",
    "drawer_close_button",
    "drawer_content",
    "drawer_footer",
    "drawer_header",
    "drawer_overlay",
    "editable",
    "editable_input",
    "editable_preview",
    "editable_textarea",
    "editor",
    "email",
    "fade",
    "flex",
    "foreach",
    "form",
    "form_control",
    "form_error_message",
    "form_helper_text",
    "form_label",
    "fragment",
    "grid",
    "grid_item",
    "heading",
    "highlight",
    "hstack",
    "html",
    "icon",
    "icon_button",
    "image",
    "input",
    "input_group",
    "input_left_addon",
    "input_left_element",
    "input_right_addon",
    "input_right_element",
    "kbd",
    "link",
    "link_box",
    "link_overlay",
    "list",
    "list_item",
    "markdown",
    "menu",
    "menu_button",
    "menu_divider",
    "menu_group",
    "menu_item",
    "menu_item_option",
    "menu_list",
    "menu_option_group",
    "modal",
    "modal_body",
    "modal_close_button",
    "modal_content",
    "modal_footer",
    "modal_header",
    "modal_overlay",
    "moment",
    "multi_select",
    "multi_select_option",
    "next_link",
    "number_decrement_stepper",
    "number_increment_stepper",
    "number_input",
    "number_input_field",
    "number_input_stepper",
    "option",
    "ordered_list",
    "password",
    "pin_input",
    "pin_input_field",
    "plotly",
    "popover",
    "popover_anchor",
    "popover_arrow",
    "popover_body",
    "popover_close_button",
    "popover_content",
    "popover_footer",
    "popover_header",
    "popover_trigger",
    "progress",
    "radio",
    "radio_group",
    "range_slider",
    "range_slider_filled_track",
    "range_slider_thumb",
    "range_slider_track",
    "responsive_grid",
    "scale_fade",
    "script",
    "select",
    "skeleton",
    "skeleton_circle",
    "skeleton_text",
    "slide",
    "slide_fade",
    "slider",
    "slider_filled_track",
    "slider_mark",
    "slider_thumb",
    "slider_track",
    "spacer",
    "span",
    "spinner",
    "square",
    "stack",
    "stat",
    "stat_arrow",
    "stat_group",
    "stat_help_text",
    "stat_label",
    "stat_number",
    "step",
    "step_description",
    "step_icon",
    "step_indicator",
    "step_number",
    "step_separator",
    "step_status",
    "step_title",
    "stepper",
    "switch",
    "tab",
    "tab_list",
    "tab_panel",
    "tab_panels",
    "table",
    "table_caption",
    "table_container",
    "tabs",
    "tag",
    "tag_close_button",
    "tag_label",
    "tag_left_icon",
    "tag_right_icon",
    "tbody",
    "td",
    "text",
    "text_area",
    "tfoot",
    "th",
    "thead",
    "tooltip",
    "tr",
    "unordered_list",
    "upload",
    "video",
    "visually_hidden",
    "vstack",
    "wrap",
    "wrap_item",
    "cancel_upload",
    "components",
    "color_mode_cond",
    "desktop_only",
    "mobile_only",
    "tablet_only",
    "mobile_and_tablet",
    "tablet_and_desktop",
    "selected_files",
    "clear_selected_files",
    "EditorButtonList",
    "EditorOptions",
    "NoSSRComponent",
    "chakra",
    "next",
    "memo",
    "el",
    "MomentDelta",
    "recharts",
    "page",
    "color_mode",
    "style",
    "toggle_color_mode",
    "utils",
    ]

)


class NextpyAttributeChecker(BaseChecker):
    """Checker class for validating nextpy attribute usage."""

    name = "nextpy-attribute-checker"
    msgs = {
        "E9008": (
            'Usage of a non-existent attribute "%s" in nextpy module. Did you mean "%s"?',
            "non-existent-nextpy-attribute",
            "Used when an attribute is accessed on the nextpy module that is not part of the allowed list of attributes.",
        ),
    }
    priority = -1

    def __init__(self, linter: Optional["PyLinter"] = None) -> None:
        """Initialize the checker.

        Args:
            linter: The Pylinter instance.
        """
        if linter is None:
            raise ValueError("A PyLinter instance is required for NextpyAttributeChecker")
        super().__init__(linter)
        self.allowed_attributes = VALID_ATTRIBUTES


    def visit_attribute(self, node: astroid.Attribute) -> None:
        """Visit an attribute node and check for invalid nextpy attributes.

        Args:
            node: The attribute node.
        """
        if isinstance(node.expr, astroid.Name) and (
            node.expr.name == "nextpy" or node.expr.name == "xt"
        ):
            # Check if node.attrname is None and handle it appropriately
            if node.attrname is None:
                return

            # Now we know node.attrname is not None and can proceed
            if node.attrname not in self.allowed_attributes:
                closest_match = self.find_closest_attribute(node.attrname)
                self.add_message(
                    "non-existent-nextpy-attribute",
                    node=node,
                    args=(node.attrname, closest_match),
                )

    def find_closest_attribute(self, attr_name: str) -> str:
        """Find closest matching attribute to the given attribute name.

        Args:
            attr_name: The attribute name to find a close match for.

        Returns:
            The closest matching attribute name if found, else a default string.
        """
        closest_matches = difflib.get_close_matches(
            attr_name, self.allowed_attributes, n=1
        )
        return closest_matches[0] if closest_matches else "no similar attribute found"


def register(linter: "PyLinter") -> None:
    """Register the checker to Pylint."""
    # print("Registering NextpyAttributeChecker.")
    linter.register_checker(NextpyAttributeChecker(linter))
